@@ -10,6 +10,7 @@ from matplotlib import rcParams
 
 from mplsoccer import dimensions
 from mplsoccer.cm import grass_cmap
+from mplsoccer.grid import _grid_dimensions, _draw_grid, grid_dimensions
 from mplsoccer.utils import Standardizer, set_visible
 
 _BinnedStatisticResult = namedtuple('BinnedStatisticResult',
@@ -33,11 +34,17 @@ class BasePitch(ABC):
         To remove the background set to "None" or 'None'.
     line_color : any Matplotlib color, default None
         The line color for the pitch markings. If None, defaults to rcParams["grid.color"].
+    line_alpha : float, default 1
+        The transparency of the pitch and the markings.
     linewidth : float, default 2
         The line width for the pitch markings.
+    linestyle : str or typle
+        Linestyle for the pitch lines:
+        {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}
+        see: https://matplotlib.org/stable/gallery/lines_bars_and_markers/linestyles.html
     line_zorder : float, default 0.9
         Set the zorder for the pitch lines (a matplotlib artist).
-         Artists with lower zorder values are drawn first.
+        Artists with lower zorder values are drawn first.
     spot_scale : float, default 0.002
         The size of the penalty and center spots relative to the pitch length.
     stripe : bool, default False
@@ -86,40 +93,31 @@ class BasePitch(ABC):
         Whether to display the goals as a 'line', 'box', 'circle' or to not display it at all (None)
     goal_alpha : float, default 1
         The transparency of the goal.
-    line_alpha : float, default 1
-        The transparency of the pitch and the markings.
+    goal_linestyle : str or typle
+        Linestyle for the pitch lines:
+        {'-', '--', '-.', ':', '', (offset, on-off-seq), ...}
+        see: https://matplotlib.org/stable/gallery/lines_bars_and_markers/linestyles.html
     axis : bool, default False
         Whether to set the axis spines to visible.
     label : bool, default False
         Whether to include the axis labels.
     tick : bool, default False
         Whether to include the axis ticks.
-    figsize : deprecated, default None
-        The figsize argument has been moved to the draw method.
-    tight_layout : deprecated, default None
-        The tight_layout argument has been moved to the draw method.
-    constrained_layout : deprecated, default None
-        The constrained_layout argument has been moved to the draw method.
-    layout : deprecated, default None
-        Layout is deprecated. Please use nrows and ncols arguments of the draw method instead.
-    view : deprecated, default None
-        View is deprecated. Please use half=True or half=False arguments instead.
-    orientation : deprecated, default None
-        Orientation is deprecated. Please use the VerticalPitch class instead:
-        from mplsoccer import VerticalPitch.
+    corner_arcs : bool, default False
+        Whether to include corner arcs.
     """
 
     def __init__(self, pitch_type='statsbomb', half=False,
-                 pitch_color=None, line_color=None, linewidth=2, line_zorder=0.9, spot_scale=0.002,
+                 pitch_color=None, line_color=None, line_alpha=1, linewidth=2,
+                 linestyle=None, line_zorder=0.9, spot_scale=0.002,
                  stripe=False, stripe_color='#c2d59d', stripe_zorder=0.6,
                  pad_left=None, pad_right=None, pad_bottom=None, pad_top=None,
                  positional=False, positional_zorder=0.8, positional_linewidth=None,
                  positional_linestyle=None, positional_color='#eadddd',
                  shade_middle=False, shade_color='#f2f2f2', shade_zorder=0.7,
-                 pitch_length=None, pitch_width=None, goal_type='line', goal_alpha=1,
-                 line_alpha=1, axis=False, label=False, tick=False,
-                 figsize=None, tight_layout=None, constrained_layout=None,
-                 layout=None, view=None, orientation=None):
+                 pitch_length=None, pitch_width=None,
+                 goal_type='line', goal_alpha=1, goal_linestyle=None,
+                 axis=False, label=False, tick=False, corner_arcs=False):
 
         # initialize attributes
         self.pitch_type = pitch_type
@@ -131,6 +129,7 @@ class BasePitch(ABC):
         if self.line_color is None:
             self.line_color = rcParams["grid.color"]
         self.linewidth = linewidth
+        self.linestyle = linestyle
         self.spot_scale = spot_scale
         self.line_zorder = line_zorder
         self.stripe = stripe
@@ -154,10 +153,12 @@ class BasePitch(ABC):
         self.pitch_width = pitch_width
         self.goal_type = goal_type
         self.goal_alpha = goal_alpha
+        self.goal_linestyle = goal_linestyle
         self.line_alpha = line_alpha
         self.axis = axis
         self.label = label
         self.tick = tick
+        self.corner_arcs = corner_arcs
 
         # other attributes for plotting circles - completed by
         # _init_circles_and_arcs / _init_circles_and_arcs_equal_aspect
@@ -165,6 +166,8 @@ class BasePitch(ABC):
         self.diameter2 = None
         self.diameter_spot1 = None
         self.diameter_spot2 = None
+        self.diameter_corner1 = None
+        self.diameter_corner2 = None
         self.arc1_theta1 = None
         self.arc1_theta2 = None
         self.arc2_theta1 = None
@@ -180,33 +183,12 @@ class BasePitch(ABC):
         self.hex_extent = None
         self.vertical = None
         self.reverse_cmap = None
-        self.standardizer = Standardizer(pitch_from=pitch_type, width_from=pitch_width,
-                                         length_from=pitch_length, pitch_to='uefa')
-
-        # deprecation warnings
-        if figsize is not None:
-            msg = "figsize has moved to the draw method."
-            warnings.warn(msg)
-        if tight_layout is not None:
-            msg = "tight_layout has moved to the draw method."
-            warnings.warn(msg)
-        if constrained_layout is not None:
-            msg = "constrained_layout has moved to the draw method."
-            warnings.warn(msg)
-        if layout is not None:
-            msg = "layout is deprecated. Please use nrows and ncols arguments instead."
-            warnings.warn(msg)
-        if view is not None:
-            msg = "view is deprecated. Please use half=True or half=False arguements instead."
-            warnings.warn(msg)
-        if orientation is not None:
-            msg = ("orientation is deprecated. Please use the VerticalPitch class instead: "
-                   "from mplsoccer import VerticalPitch.")
-            warnings.warn(msg)
 
         # data checks
         self._validation_checks()
 
+        self.standardizer = Standardizer(pitch_from=pitch_type, width_from=pitch_width,
+                                         length_from=pitch_length, pitch_to='uefa')
         # set pitch dimensions
         self.dim = dimensions.create_pitch_dims(pitch_type, pitch_width, pitch_length)
 
@@ -251,6 +233,7 @@ class BasePitch(ABC):
                 f'pitch_type={self.pitch_type!r}, half={self.half!r}, '
                 f'pitch_color={self.pitch_color!r}, line_color={self.line_color!r}, '
                 f'linewidth={self.linewidth!r}, line_zorder={self.line_zorder!r}, '
+                f'linestyle={self.linestyle!r}, '
                 f'stripe={self.stripe!r}, stripe_color={self.stripe_color!r}, '
                 f'stripe_zorder={self.stripe_zorder!r}, '
                 f'pad_left={self.pad_left!r}, pad_right={self.pad_right!r}, '
@@ -263,7 +246,9 @@ class BasePitch(ABC):
                 f'pitch_length={self.pitch_length!r}, pitch_width={self.pitch_width!r}, '
                 f'goal_type={self.goal_type!r}, goal_alpha={self.goal_alpha!r}, '
                 f'line_alpha={self.line_alpha!r}, label={self.label!r}, '
-                f'tick={self.tick!r}, axis={self.axis!r}, spot_scale={self.spot_scale!r})')
+                f'tick={self.tick!r}, axis={self.axis!r}, spot_scale={self.spot_scale!r}, '
+                f'corner_arcs={self.corner_arcs!r})'
+                )
 
     def _validation_checks(self):
         # pitch validation
@@ -308,6 +293,8 @@ class BasePitch(ABC):
         self.diameter2 = self.dim.circle_diameter
         self.diameter_spot1 = self.spot_scale * self.dim.length * 2
         self.diameter_spot2 = self.spot_scale * self.dim.length * 2
+        self.diameter_corner1 = self.dim.corner_diameter
+        self.diameter_corner2 = self.dim.corner_diameter
         self.arc1_theta1 = -self.dim.arc
         self.arc1_theta2 = self.dim.arc
         self.arc2_theta1 = 180 - self.dim.arc
@@ -332,9 +319,9 @@ class BasePitch(ABC):
         radius_length = radius * self.dim.length / self.dim.pitch_length
         radius_width = radius * self.dim.width / self.dim.pitch_width
         intersection = self.dim.center_width - ((radius_width * radius_length *
-                                                (radius_length ** 2 -
-                                                 (self.dim.penalty_area_length -
-                                                  self.dim.penalty_left) ** 2) ** 0.5) /
+                                                 (radius_length ** 2 -
+                                                  (self.dim.penalty_area_length -
+                                                   self.dim.penalty_left) ** 2) ** 0.5) /
                                                 radius_length ** 2)
         arc_pen_top1 = (self.dim.penalty_area_length, intersection)
         spot_xy = (self.dim.penalty_left, self.dim.center_width)
@@ -351,6 +338,7 @@ class BasePitch(ABC):
 
     def _init_circles_and_arcs_equal_aspect(self, ax):
         radius_center = self.dim.circle_diameter / 2
+        radius_corner = self.dim.corner_diameter / 2
         radius_spot = self.spot_scale * self.dim.pitch_length
 
         (self.diameter1,
@@ -361,6 +349,12 @@ class BasePitch(ABC):
          self.diameter_spot2) = self._diameter_circle_equal_aspect(self.dim.penalty_left,
                                                                    self.dim.center_width,
                                                                    ax, radius_spot)
+
+        (self.diameter_corner1,
+         self.diameter_corner2) = self._diameter_circle_equal_aspect(self.dim.left,
+                                                                     self.dim.bottom,
+                                                                     ax, radius_corner)
+
         self._arc_angles_equal_aspect(ax, radius_center)
 
     @staticmethod
@@ -468,30 +462,54 @@ class BasePitch(ABC):
                 self._draw_stripe(ax, i)
 
     def _draw_pitch_markings(self, ax):
+        # if we use rectangles here then the linestyle isn't consistent around the pitch
+        # as sometimes the rectangles overlap with each other and the gaps between
+        # lines can when they overlap can look like a solid line even with -. linestyles.
         line_prop = {'linewidth': self.linewidth, 'alpha': self.line_alpha,
-                     'color': self.line_color, 'zorder': self.line_zorder}
-        # draw lines as one-continous loop so that alpha (transparency) values do not overlap
-        xs = [self.dim.center_length, self.dim.center_length, self.dim.right, self.dim.right,
-              self.dim.penalty_area_right, self.dim.penalty_area_right, self.dim.right,
-              self.dim.right, self.dim.six_yard_right, self.dim.six_yard_right, self.dim.right,
-              self.dim.right, self.dim.left, self.dim.left, self.dim.penalty_area_left,
-              self.dim.penalty_area_left, self.dim.left, self.dim.left, self.dim.six_yard_left,
-              self.dim.six_yard_left, self.dim.left, self.dim.left, self.dim.center_length, ]
-        ys = [self.dim.bottom, self.dim.top, self.dim.top, self.dim.penalty_area_bottom,
-              self.dim.penalty_area_bottom, self.dim.penalty_area_top, self.dim.penalty_area_top,
-              self.dim.six_yard_bottom, self.dim.six_yard_bottom, self.dim.six_yard_top,
-              self.dim.six_yard_top, self.dim.bottom, self.dim.bottom, self.dim.penalty_area_top,
-              self.dim.penalty_area_top, self.dim.penalty_area_bottom, self.dim.penalty_area_bottom,
-              self.dim.six_yard_top, self.dim.six_yard_top, self.dim.six_yard_bottom,
-              self.dim.six_yard_bottom, self.dim.top, self.dim.top, ]
-        self._draw_line(ax, xs, ys, **line_prop)
+                     'color': self.line_color, 'zorder': self.line_zorder,
+                     'linestyle': self.linestyle,
+                     }
+        # main markings (outside of pitch and center line)
+        xs_main = [self.dim.center_length, self.dim.center_length, self.dim.right,
+                   self.dim.right, self.dim.left, self.dim.left, self.dim.center_length,
+                   ]
+        ys_main = [self.dim.bottom, self.dim.top, self.dim.top,
+                   self.dim.bottom, self.dim.bottom, self.dim.top, self.dim.top,
+                   ]
+        self._draw_line(ax, xs_main, ys_main, **line_prop)
+        # penalty boxs
+        xs_pbox_right = [self.dim.right, self.dim.penalty_area_right,
+                         self.dim.penalty_area_right, self.dim.right,
+                         ]
+        xs_pbox_left = [self.dim.left, self.dim.penalty_area_left,
+                        self.dim.penalty_area_left, self.dim.left,
+                        ]
+        ys_pbox = [self.dim.penalty_area_bottom, self.dim.penalty_area_bottom,
+                   self.dim.penalty_area_top, self.dim.penalty_area_top,
+                   ]
+        self._draw_line(ax, xs_pbox_right, ys_pbox, **line_prop)
+        self._draw_line(ax, xs_pbox_left, ys_pbox, **line_prop)
+        # six-yard box
+        xs_sixbox_right = [self.dim.right, self.dim.six_yard_right,
+                           self.dim.six_yard_right, self.dim.right,
+                           ]
+        xs_sixbox_left = [self.dim.left, self.dim.six_yard_left,
+                          self.dim.six_yard_left, self.dim.left,
+                          ]
+        ys_sixbox = [self.dim.six_yard_bottom, self.dim.six_yard_bottom,
+                     self.dim.six_yard_top, self.dim.six_yard_top,
+                     ]
+        self._draw_line(ax, xs_sixbox_right, ys_sixbox, **line_prop)
+        self._draw_line(ax, xs_sixbox_left, ys_sixbox, **line_prop)
         self._draw_circles_and_arcs(ax)
 
     def _draw_circles_and_arcs(self, ax):
         circ_prop = {'fill': False, 'linewidth': self.linewidth, 'alpha': self.line_alpha,
-                     'color': self.line_color, 'zorder': self.line_zorder}
+                     'color': self.line_color, 'zorder': self.line_zorder,
+                     'linestyle': self.linestyle,
+                     }
 
-        # draw center cicrle and penalty area arcs
+        # draw center circle and penalty area arcs
         self._draw_ellipse(ax, self.dim.center_length, self.dim.center_width,
                            self.diameter1, self.diameter2, **circ_prop)
         self._draw_arc(ax, self.dim.penalty_left, self.dim.center_width,
@@ -501,6 +519,22 @@ class BasePitch(ABC):
                        self.diameter1, self.diameter2,
                        theta1=self.arc2_theta1, theta2=self.arc2_theta2, **circ_prop)
 
+        if self.corner_arcs:
+            if self.dim.invert_y:
+                thetas = [(0, 90), (90, 180), (180, 270), (270, 360)]
+            else:
+                thetas = [(270, 360), (180, 270), (90, 180), (0, 90)]
+            if self.vertical:
+                thetas = np.flip(thetas, axis=0)
+            corner_points = [(self.dim.left, self.dim.top),
+                             (self.dim.right, self.dim.top),
+                             (self.dim.right, self.dim.bottom),
+                             (self.dim.left, self.dim.bottom)]
+            for i, (x, y) in enumerate(corner_points):
+                t1, t2 = thetas[i]
+                self._draw_arc(ax, x, y, self.diameter_corner1, self.diameter_corner2,
+                               theta1=t1, theta2=t2, **circ_prop)
+
         # draw center and penalty spots
         if self.spot_scale > 0:
             self._draw_ellipse(ax, self.dim.center_length, self.dim.center_width,
@@ -518,18 +552,33 @@ class BasePitch(ABC):
 
     def _draw_goals(self, ax):
         if self.goal_type == 'box':
-            rect_prop = {'fill': False, 'linewidth': self.linewidth, 'color': self.line_color,
-                         'alpha': self.goal_alpha, 'zorder': self.line_zorder}
-            self._draw_rectangle(ax, self.dim.right, self.dim.goal_bottom,
-                                 self.dim.goal_length, self.dim.goal_width, **rect_prop)
-            self._draw_rectangle(ax, self.dim.left - self.dim.goal_length,
-                                 self.dim.goal_bottom, self.dim.goal_length,
-                                 self.dim.goal_width, **rect_prop)
+            line_prop = {'linewidth': self.linewidth, 'color': self.line_color,
+                         'alpha': self.goal_alpha, 'zorder': self.line_zorder,
+                         'linestyle': self.goal_linestyle,
+                         }
+            # left goal
+            xs_left = [self.dim.left, self.dim.left - self.dim.goal_length,
+                       self.dim.left - self.dim.goal_length, self.dim.left,
+                       ]
+            ys_left = [self.dim.goal_bottom, self.dim.goal_bottom,
+                       self.dim.goal_top, self.dim.goal_top,
+                       ]
+
+            self._draw_line(ax, xs_left, ys_left, **line_prop)
+            # right goal
+            xs_right = [self.dim.right, self.dim.right + self.dim.goal_length,
+                        self.dim.right + self.dim.goal_length, self.dim.right,
+                        ]
+            ys_right = [self.dim.goal_bottom, self.dim.goal_bottom,
+                        self.dim.goal_top, self.dim.goal_top,
+                        ]
+            self._draw_line(ax, xs_right, ys_right, **line_prop)
 
         elif self.goal_type == 'line':
-            goal_linewidth = self.linewidth * 2
-            line_prop = {'linewidth': goal_linewidth, 'color': self.line_color,
-                         'alpha': self.goal_alpha, 'zorder': self.line_zorder}
+            line_prop = {'linewidth': self.linewidth * 2, 'color': self.line_color,
+                         'alpha': self.goal_alpha, 'zorder': self.line_zorder,
+                         'linestyle': self.goal_linestyle,
+                         }
             self._draw_line(ax, [self.dim.right, self.dim.right],
                             [self.dim.goal_top, self.dim.goal_bottom], **line_prop)
             self._draw_line(ax, [self.dim.left, self.dim.left],
@@ -548,8 +597,13 @@ class BasePitch(ABC):
                      'alpha': self.line_alpha, 'linestyle': self.positional_linestyle,
                      'zorder': self.positional_zorder}
         # x lines for Juego de Posición
-        for coord in self.dim.positional_x[1:-1]:
-            self._draw_line(ax, [coord, coord], [self.dim.bottom, self.dim.top], **line_prop)
+        #through lines
+        self._draw_line(ax, [self.dim.positional_x[1], self.dim.positional_x[1]], [self.dim.bottom, self.dim.top], **line_prop)
+        self._draw_line(ax, [self.dim.positional_x[5], self.dim.positional_x[5]], [self.dim.bottom, self.dim.top], **line_prop)
+        #short lines
+        for coord in self.dim.positional_x[2:5]:
+            self._draw_line(ax, [coord, coord], [self.dim.bottom,  self.dim.penalty_area_bottom], **line_prop)
+            self._draw_line(ax, [coord, coord], [self.dim.top,  self.dim.penalty_area_top], **line_prop)
         # y lines for Juego de Posición
         self._draw_line(ax, [self.dim.left, self.dim.right],
                         [self.dim.positional_y[1], self.dim.positional_y[1]], **line_prop)
@@ -567,9 +621,9 @@ class BasePitch(ABC):
                              self.dim.positional_x[4] - self.dim.positional_x[2], self.dim.width,
                              **shade_prop)
 
-    def grid(self, figheight=9, nrows=1, ncols=1, left=None, grid_width=0.95,
-             bottom=None, endnote_height=0.065, endnote_space=0.01,
-             grid_height=0.715, title_space=0.01, title_height=0.15, space=0.05, axis=True):
+    def grid(self, figheight=9, nrows=1, ncols=1, grid_height=0.715, grid_width=0.95, space=0.05,
+             left=None, bottom=None, endnote_height=0.065, endnote_space=0.01,
+             title_height=0.15, title_space=0.01, axis=True):
         """ A helper to create a grid of pitches in a specified location
 
         Parameters
@@ -578,12 +632,19 @@ class BasePitch(ABC):
             The figure height in inches.
         nrows, ncols : int, default 1
             Number of rows/columns of pitches in the grid.
-        left : float, default None
-            The location of the left hand side of the axes in fractions of the figure width.
-            The default of None places the axes in the middle of the figure.
+        grid_height : float, default 0.715
+            The height of the pitch grid in fractions of the figure height.
+            The default is the grid height is 71.5% of the figure height.
         grid_width : float, default 0.95
             The width of the pitch grid in fractions of the figure width.
             The default is the grid is 95% of the figure width.
+        space : float, default 0.05
+            The total amount of the grid height reserved for spacing between the pitch axes.
+            Expressed as a fraction of the grid_height. The default is 5% of the grid height.
+            The spacing across the grid width is automatically calculated to maintain even spacing.
+        left : float, default None
+            The location of the left-hand side of the axes in fractions of the figure width.
+            The default of None places the axes in the middle of the figure.
         bottom : float, default None
             The location of the bottom endnote axes in fractions of the figure height.
             The default of None places the axes in the middle of the figure.
@@ -596,21 +657,14 @@ class BasePitch(ABC):
             The space between the pitch grid and endnote axis in fractions of the figure height.
             The default space is 1% of the figure height.
             If endnote_height=0, then the endnote_space is set to zero.
-        grid_height : float, default 0.715
-            The height of the pitch grid in fractions of the figure height.
-            The default is the grid height is 71.5% of the figure height.
-        title_space : float, default 0.01
-            The space between the pitch grid and title axis in fractions of the figure height.
-            The default space is 1% of the figure height.
-            If title_height=0, then the title_space is set to zero.
         title_height : float, default 0.15
             The height of the title axis in fractions of the figure height.
             The default is the title axis is 15% of the figure height.
             If title_height=0, then the title axes is not plotted.
-        space : float, default 0.05
-            The total amount of the grid height reserved for spacing between the pitch axes.
-            Expressed as a fraction of the grid_height. The default is 5% of the grid height.
-            The spacing across the grid width is automatically calculated to maintain even spacing.
+        title_space : float, default 0.01
+            The space between the pitch grid and title axis in fractions of the figure height.
+            The default space is 1% of the figure height.
+            If title_height=0, then the title_space is set to zero.
         axis : bool, default True
             Whether the endnote and title axes are 'on'.
 
@@ -627,113 +681,28 @@ class BasePitch(ABC):
         >>> pitch = Pitch()
         >>> fig, axs = pitch.grid(nrows=3, ncols=3, grid_height=0.7, figheight=14)
         """
-        if left is None:
-            left = (1 - grid_width) / 2
-
-        if title_height == 0:
-            title_space = 0
-
-        if endnote_height == 0:
-            endnote_space = 0
-
-        error_msg_height = ('The axes extends past the figure height. '
-                            'Reduce one of the bottom, endnote_height, endnote_space, grid_height, '
-                            'title_space or title_height so the total is ≤ 1.')
-        error_msg_width = ('The grid axes extends past the figure width. '
-                           'Reduce one of the grid_width or left so the total is ≤ 1.')
-
-        axes_height = endnote_height + endnote_space + grid_height + title_space + title_height
-        if axes_height > 1:
-            raise ValueError(error_msg_height)
-
-        if bottom is None:
-            bottom = (1 - axes_height) / 2
-
-        if bottom + axes_height > 1:
-            raise ValueError(error_msg_height)
-
-        if grid_width + left > 1:
-            raise ValueError(error_msg_width)
-
-        # calculate the figure width
-        if (nrows > 1) and (ncols > 1):
-            figwidth = figheight * grid_height / grid_width * (((1 - space) * self.ax_aspect *
-                                                                ncols / nrows) +
-                                                               (space * (ncols - 1) / (nrows - 1)))
-            individual_space_height = grid_height * space / (nrows - 1)
-            individual_space_width = individual_space_height * figheight / figwidth
-            individual_pitch_height = grid_height * (1 - space) / nrows
-
-        elif (nrows > 1) and (ncols == 1):
-            figwidth = grid_height * figheight / grid_width * (1 - space) * self.ax_aspect / nrows
-            individual_space_height = grid_height * space / (nrows - 1)
-            individual_space_width = 0
-            individual_pitch_height = grid_height * (1 - space) / nrows
-
-        elif (nrows == 1) and (ncols > 1):
-            figwidth = grid_height * figheight / grid_width * (space + self.ax_aspect * ncols)
-            individual_space_height = 0
-            individual_space_width = grid_height * space * figheight / figwidth / (ncols - 1)
-            individual_pitch_height = grid_height
-
-        else:  # nrows=1, ncols=1
-            figwidth = grid_height * self.ax_aspect * figheight / grid_width
-            individual_space_height = 0
-            individual_space_width = 0
-            individual_pitch_height = grid_height
-
-        individual_pitch_width = individual_pitch_height * self.ax_aspect * figheight / figwidth
-
-        bottom_coordinates = np.tile(individual_space_height + individual_pitch_height,
-                                     reps=nrows - 1).cumsum()
-        bottom_coordinates = np.insert(bottom_coordinates, 0, 0.)
-        bottom_coordinates = np.repeat(bottom_coordinates, ncols)
-        grid_bottom = bottom + endnote_height + endnote_space
-        bottom_coordinates = bottom_coordinates + grid_bottom
-        bottom_coordinates = bottom_coordinates[::-1]
-
-        left_coordinates = np.tile(individual_space_width + individual_pitch_width,
-                                   reps=ncols - 1).cumsum()
-        left_coordinates = np.insert(left_coordinates, 0, 0.)
-        left_coordinates = np.tile(left_coordinates, nrows)
-        left_coordinates = left_coordinates + left
-
-        fig = plt.figure(figsize=(figwidth, figheight))
-        axs = []
-        for idx, bottom_coord in enumerate(bottom_coordinates):
-            axs.append(fig.add_axes((left_coordinates[idx], bottom_coord,
-                                     individual_pitch_width, individual_pitch_height)))
-            self.draw(ax=axs[idx])
-        axs = np.squeeze(np.array(axs).reshape((nrows, ncols)))
-        if axs.size == 1:
-            axs = axs.item()
-
+        dim = _grid_dimensions(ax_aspect=self.ax_aspect, figheight=figheight, nrows=nrows,
+                               ncols=ncols, grid_height=grid_height, grid_width=grid_width,
+                               space=space, left=left, bottom=bottom,
+                               endnote_height=endnote_height, endnote_space=endnote_space,
+                               title_height=title_height, title_space=title_space)
         left_pad = (np.abs(self.visible_pitch - self.extent)[0] /
-                    np.abs(self.extent[1] - self.extent[0])) * individual_pitch_width
+                    np.abs(self.extent[1] - self.extent[0])) * dim['axwidth']
         right_pad = (np.abs(self.visible_pitch - self.extent)[1] /
-                     np.abs(self.extent[1] - self.extent[0])) * individual_pitch_width
-        title_left = left + left_pad
-        title_width = grid_width - left_pad - right_pad
+                     np.abs(self.extent[1] - self.extent[0])) * dim['axwidth']
+        fig, axs = _draw_grid(dimensions=dim, left_pad=left_pad, right_pad=right_pad,
+                              axis=axis, grid_key='pitch')
 
-        result_axes = {'pitch': axs}
+        if endnote_height > 0 or title_height > 0:
+            for ax in np.asarray(axs['pitch']).flat:
+                self.draw(ax=ax)
+        else:
+            for ax in np.asarray(axs).flat:
+                self.draw(ax=ax)
 
-        if title_height > 0:
-            ax_title = fig.add_axes((title_left, grid_bottom + grid_height + title_space,
-                                     title_width, title_height))
-            if axis is False:
-                ax_title.axis('off')
-            result_axes['title'] = ax_title
+        return fig, axs
 
-        if endnote_height > 0:
-            ax_endnote = fig.add_axes((title_left, bottom,
-                                       title_width, endnote_height))
-            if axis is False:
-                ax_endnote.axis('off')
-            result_axes['endnote'] = ax_endnote
-
-        return fig, result_axes
-
-    def calculate_grid_dimensions(self, figwidth, figheight, nrows, ncols, max_grid, space):
+    def grid_dimensions(self, figwidth, figheight, nrows, ncols, max_grid, space):
         """ A helper method to propose a grid_width and grid_height for grid based on the inputs.
 
         Parameters
@@ -757,33 +726,15 @@ class BasePitch(ABC):
         --------
         >>> from mplsoccer import Pitch
         >>> pitch = Pitch()
-        >>> grid_width, grid_height = pitch.calculate_grid_dimensions(figwidth=16, figheight=9, \
-                                                                      nrows=1, ncols=1, \
-                                                                      max_grid=1,  space=0)
+        >>> grid_width, grid_height = pitch.grid_dimensions(figwidth=16, figheight=9, \
+                                                            nrows=1, ncols=1, \
+                                                            max_grid=1,  space=0)
         """
-        # grid1 = calculate the grid_width given the max_grid as grid_height
-        # grid2 = calculate grid_height given the max_grid as grid_width
-        if (nrows > 1) and (ncols > 1):
-            grid1 = max_grid * figheight / figwidth * (((1 - space) * self.ax_aspect *
-                                                        ncols / nrows) +
-                                                       (space * (ncols - 1) / (nrows - 1)))
-            grid2 = max_grid / figheight * figwidth / (((1 - space) * self.ax_aspect *
-                                                        ncols / nrows) +
-                                                       (space * (ncols - 1) / (nrows - 1)))
-        elif (nrows > 1) and (ncols == 1):
-            grid1 = max_grid * figheight / figwidth * (1 - space) * self.ax_aspect / nrows
-            grid2 = max_grid / figheight * figwidth / (1 - space) * self.ax_aspect / nrows
-        elif (nrows == 1) and (ncols > 1):
-            grid1 = max_grid * figheight / figwidth * (space + self.ax_aspect * ncols)
-            grid2 = max_grid / figheight * figwidth / (space + self.ax_aspect * ncols)
-        else:
-            grid1 = max_grid * figheight / figwidth * self.ax_aspect
-            grid2 = max_grid / figheight * figwidth / self.ax_aspect
-
-        # decide whether the max_grid is the grid_width or grid_height and set the other value
-        if (grid1 > 1) | ((grid2 >= grid1) & (grid2 <= 1)):
-            return max_grid, grid2
-        return grid1, max_grid
+        grid_width, grid_height = grid_dimensions(self.ax_aspect, figwidth=figwidth,
+                                                  figheight=figheight,
+                                                  nrows=nrows, ncols=ncols,
+                                                  max_grid=max_grid, space=space)
+        return grid_width, grid_height
 
     def jointgrid(self, figheight=9, left=None, grid_width=0.95,
                   bottom=None, endnote_height=0.065, endnote_space=0.01,
@@ -798,7 +749,7 @@ class BasePitch(ABC):
         figheight : float, default 9
             The figure height in inches.
         left : float, default None
-            The location of the left hand side of the grid in fractions of the figure width.
+            The location of the left-hand side of the grid in fractions of the figure width.
             The default of None places the axes in the middle of the figure.
         grid_width : float, default 0.95
             The width of the grid area in fractions of the figure width.
@@ -1023,7 +974,7 @@ class BasePitch(ABC):
 
         return fig, axs
 
-    # The methods below for drawing/ setting attributes for some of the pitch elements
+    # The methods below for drawing/ setting attributes for some pitch elements
     # are defined in pitch.py (Pitch/ VerticalPitch classes)
     # as they differ for horizontal/ vertical pitches
     @abstractmethod
@@ -1033,23 +984,23 @@ class BasePitch(ABC):
     @abstractmethod
     def _set_extent(self):
         """ Implement a method to set the pitch extents, stripe locations,
-         and attributes to help plotting on differnt orientations."""
+         and attributes to help plot on different orientations."""
 
     @abstractmethod
     def _draw_rectangle(self, ax, x, y, width, height, **kwargs):
-        """ Implement a method to draw rectangles on a axes."""
+        """ Implement a method to draw rectangles on an axes."""
 
     @abstractmethod
     def _draw_line(self, ax, x, y, **kwargs):
-        """ Implement a method to draw lines on a axes."""
+        """ Implement a method to draw lines on an axes."""
 
     @abstractmethod
     def _draw_ellipse(self, ax, x, y, width, height, **kwargs):
-        """ Implement a method to draw ellipses (circles) on a axes."""
+        """ Implement a method to draw ellipses (circles) on an axes."""
 
     @abstractmethod
     def _draw_arc(self, ax, x, y, width, height, theta1, theta2, **kwargs):
-        """ Implement a method to draw arcs on a axes."""
+        """ Implement a method to draw arcs on an axes."""
 
     @abstractmethod
     def _draw_stripe(self, ax, i):
@@ -1121,7 +1072,7 @@ class BasePitch(ABC):
 
     @abstractmethod
     def bin_statistic_positional(self, x, y, values=None, positional='full',
-                                 normalize=False, statistic='count'):
+                                 statistic='count', normalize=False):
         """ Calculate the binned statistics for Juegos de posición zones."""
 
     @abstractmethod
@@ -1129,7 +1080,7 @@ class BasePitch(ABC):
         """ Implement a heatmap for the Juegos de posición zones."""
 
     @abstractmethod
-    def label_heatmap(self, stats, ax=None, **kwargs):
+    def label_heatmap(self, stats, str_format=None, exclude_zeros=False, ax=None, **kwargs):
         """ Implement a heatmap labeller."""
 
     @abstractmethod
@@ -1160,3 +1111,7 @@ class BasePitch(ABC):
              color=None, ax=None, **kwargs):
         """ Implement a flow diagram with arrows showing the average direction and
         a heatmap showing the counts in each bin."""
+
+    @abstractmethod
+    def triplot(self, x, y, ax=None, **kwargs):
+        """ Implement a wrapper for matplotlib.axes.Axes.triplot."""
